
#ifndef lint
static const char revid[] = 
	"@(#)$Id: damldb_admin.cpp,v 1.2 2002/04/09 22:05:57 neves Exp $";
#endif // not lint

#include <damldb/DAMLDB.hh>
#include <damldb/IDamlDB.hh>
#include <damldb/StatementIterator.hh>
#include <damldb/Timer.hh>

#include "db_admin_cmdline.h"

#include <algorithm>
#include <cstdlib>
#include <iostream>
#include <stdexcept>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  char const* const kRDFS_CLASS_URI	= "http://www.w3.org/2000/01/rdf-schema#Class";
  char const* const kRDFTYPE_URI	= "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
  const int	    kOPEN_MODE		= iodb::in | iodb::out | iodb::nocreate;

  ////////////////////////////////////////////////////////////

  class list_classes_functor
  {
  public:
    
    static unsigned int count;

  public:
    list_classes_functor(ostream& os,
			 IDamlDB& db,
			 const ResourceID rdfsClass,
			 const ResourceID rdfType,
			 bool print,
			 bool translate) :
      db_(db),
      rdfsClass_(rdfsClass),
      rdfType_(rdfType),
      print_(print),
      translate_(translate),
      os_(os)
    {}

    int operator() (const Statement& statement) const
    {
      if ((statement.object == rdfsClass_) &&
	  (statement.predicate == rdfType_))
	{
	  if (translate_)
	    {
	      string tmpStr;
	      const string& uri = db_.resource2uri(statement.subject, tmpStr);
	      if (print_)
		{ os_ << "  " << uri << endl; }
	    }
	  else
	    { count++; }

	}

      return 0;
    }

  private:

    mutable IDamlDB& db_;
    ResourceID rdfsClass_;
    ResourceID rdfType_;
    bool print_;
    bool translate_;
    mutable ostream& os_;
  };

  unsigned int  list_classes_functor::count = 0;

  ////////////////////////////////////////////////////////////

  static
  void
  dump_resources(IDamlDB& db)
  {
    db.dump_resources(cout);
  }

  ////////////////////////////////////////////////////////////

  static
  void
  dump_statements(IDamlDB& db)
  {
    db.dump_statements(cout);
  }

  ////////////////////////////////////////////////////////////

  static
  void
  list_classes(IDamlDB& db, const bool translate, const bool print)
  {
    unsigned int count = 0;

    if (print)
      {	cout << "Classes:" << endl; }

    try
      {
	const ResourceID rdfType   = db.uri2resource(kRDFTYPE_URI);
	const ResourceID rdfsClass = db.uri2resource(kRDFS_CLASS_URI);
	const Resource& rdfsClassResource = db.get_resource(rdfsClass);
	Statement statement = db.get_statement(rdfsClassResource.objectFirst);

	StatementIterator::iterator stmt_iter(&db, &statement);
	StatementIterator::iterator stmt_iter_end;
	list_classes_functor functor(cout, db, rdfsClass, rdfType, print, translate);

	for_each<StatementIterator::iterator, list_classes_functor>
	  (stmt_iter, stmt_iter_end, functor);

	count = list_classes_functor::count;
      }
    catch( const exception& e) {} // FIXME: Access denied if database is RDONLY and empty

    if (! print)
      { cout << count << " classes" << endl; }
  }

  ////////////////////////////////////////////////////////////

  static
  void
  show_statictics(IDamlDB& db)
  {
    IDamlDB::size_type nStatements = db.get_statement_count();
    IDamlDB::size_type nResources = db.get_resource_count();

    cout << "Number of Statements: " << nStatements << endl;
    cout << "Number of Resources : " << nResources << endl;
  }

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

////////////////////////////////////////////////////////////

#ifdef CCXX_NAMESPACES

/* FIXME: USING(damldb) Generates an INTERNAL COMPILER ERROR under MS VC98 -- P.N. */

#  if !defined(_MSC_VER)
     USING(damldb);
#  else
    USES(damldb,kOPEN_MODE)
    USES(damldb,iodb)
    USES(damldb,DAMLDB)
    USES(damldb,IDamlDB)
    USES(damldb,Timer)
    USES(damldb,list_classes)
    USES(damldb,dump_statements)
    USES(damldb,dump_resources)
    USES(damldb,show_statictics)
#  endif /* !_MSC_VER */

#endif /* CCXX_NAMESPACES */

int
main(int argc, char **argv)
{
  gengetopt_args_info	args_info;  
  int			exit_status = EXIT_SUCCESS;
  int			open_mode   = kOPEN_MODE;

  cmdline_parser (argc, argv, &args_info);

  try
    {
      int action_flag_count = 0;

      if (args_info.create_flag)
	{
	  open_mode = iodb::in | iodb::out;
	  action_flag_count++;
	}

      if (args_info.countClasses_flag)
	{ action_flag_count++; }

      if (args_info.dump_flag)
	{ action_flag_count++; }

      if (args_info.listClasses_flag)
	{ action_flag_count++; }

      if (args_info.statistics_flag)
	{ action_flag_count++; }

      // check flag exclusivity
      if (action_flag_count != 1)
	{
	  cmdline_parser_print_help();
	  return EXIT_FAILURE;
	}

      DAMLDB::set_open_mode(open_mode);
      IDamlDB& db = *DAMLDB::getInstance()->db;
      Timer timer;

      if (args_info.time_flag)
	{ timer.start(); }

      if (args_info.countClasses_flag)
	{ list_classes(db, false, false); }
      else if (args_info.listClasses_flag)
	{ list_classes(db, true, true);	}
      else if (args_info.dump_flag)
	{
	  dump_statements(db);
	  dump_resources(db);
	}
      else if (args_info.statistics_flag)
	{ show_statictics(db); }

      if (args_info.time_flag)
	{
	  timer.stop();

	  cout << "elapsed time = ";
	  cout << timer.elapsed_time().sec();
	  cout << ".";
	  cout << timer.elapsed_time().usec();
	  cout << " seconds ";
	  cout << endl;
	}
    }
  catch( const exception& x)
    {
      cerr << "FATAL ERROR: " << x.what() << endl;
      exit_status = EXIT_FAILURE;
    }
  catch(...)
    {
      cerr << "Unknown exception ocurred" << endl;
      exit_status = EXIT_FAILURE;
    }

  return exit_status;
}

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
