
#ifndef _DAMLDB_DAMLDB_HH
#define _DAMLDB_DAMLDB_HH

#if defined (sun)
#  pragma ident "@(#)$Id: DAMLDB.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Export.hh"
#include "damldb/IDamlDB.hh"
#include "damldb/Portability.hh"
#include "damldb/damldb_iodb.hh"

#include <string>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  /** DAMLDB class configures and obtains an instance of the damldb database.
  */
  class __EXPORT DAMLDB {
  public:

    DAMLDB();

    ~DAMLDB();

    /** Get the sole instance of the DAMLDB class.
	@return the pointer to the DAMLDB instance.
    */
    static DAMLDB * getInstance();

    /** Get the default open mode for the database.

	The open mode is specified using three open modes defined by the
	the iostream class:

	\begin{description}
	\item[iodb::in] open damldb database for reading
	\item[iodb::out] open the damldb database for writing
	\item[iodb::nocreate] open an existing damldb database. Otherwise, fail.
	\end{description}

	The default value is \emph{iodb::in | iodb::out | iodb::nocreate}.

	@return the mode to open the damldb database.
    */
    static const int get_open_mode();

    /** Set the open mode for the damldb database.

	The open mode is specified using three open modes used by the
	the iostream class: \emph{iodb::in, iodb::out, and iodb::nocreate}.

	@param open_mode the open mode of the database.
    */
    static void set_open_mode(const int open_mode);

    /** Get the data directory for the database.  

        The data directory property determines which directory will
        contain the damldb database files.

	The default value is the data directory of the installed
	damldb packages (i.e. $(datadir)/share directory). A user of
	the damldb package may override the default value using the
	environment variable \emph{DAMLDB_DATA_DIR} or by calling the
	setDataDirectory method.

	@return the path name of the directory.
	@see setDataDirectory
    */
    static const string& getDataDirectory();

    /** Set the default data directory for the database.
	@param datadir the path name of the directory
    */
    static void setDataDirectory(const string& datadir);

  public:

    IDamlDB* db;

  private:

    static DAMLDB * instance_;
    static int open_mode_;
    static string data_dir_;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_DAMLDB_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
