
#ifndef _DAMLDB_DAMLDB_IMPL_HH
#define _DAMLDB_DAMLDB_IMPL_HH

#if defined (sun)
#  pragma ident "@(#)$Id: DamlDBImpl.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/IDamlDB.hh"
#include "damldb/IntStringMap.hh"
#include "damldb/Portability.hh"
#include "damldb/RecordDB.hh"
#include "damldb/Statement.hh"
#include "damldb/StringIntMap.hh"
#include "damldb/Types.hh"

#include <string>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  class DamlDBImpl : public IDamlDB {
  public:
    typedef size_t size_type;

  public:

    DamlDBImpl(const string& directory,
		int open_mode,
		stmt_mapped_allocator& stmt_alloc,
		res_mapped_allocator& res_alloc);

    ~DamlDBImpl();

    void add_resource();

    void add_resource(const Resource& resource);

    size_type get_resource_count() const;

    const Resource& get_resource(const ResourceID& id) const;

    const Resource& get_resource(const char* uri);

    const Statement& add_statement(const char* sourceURI,
				   const char* subjectURI,
				   const char* predicateURI,
				   const char* objectURI);

    const Statement& add_statement(const ResourceID& sourceID,
				   const ResourceID& subjectID,
				   const ResourceID& predicateID,
				   const ResourceID& objectID);

    const Statement& add_statement(const Statement& statement);

    size_type get_statement_count() const;

    const Statement& get_statement(const StatementID& id) const;

    const string& resource2uri(const ResourceID& resource, string& result);

    ResourceID uri2resource(const char* uri);

    void sync();

    void dump_statements(ostream& os) const;

    void dump_resources(ostream& os) const;

  protected:

    void update_indices(const string& s, const ResourceID& id);

  private:

    StatementDB  _statementDB;
    ResourceDB   _resourceDB;
    IntStringMap _i2s;
    StringIntMap _s2i;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_DAMLDB_IMPL_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
