#ifndef _DAMLDB_MMAP_ALLOCATOR_HH
#define _DAMLDB_MMAP_ALLOCATOR_HH

#if defined (sun)
#  pragma ident "@(#)$Id: MMapAllocator.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

#include <cstddef>
#include <cstdlib>

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  template <class T> class MMapAllocator
  {
  public:
    typedef T                 value_type;
    typedef value_type*       pointer;
    typedef const value_type* const_pointer;
    typedef value_type&       reference;
    typedef const value_type& const_reference;
    typedef size_t            size_type;
    typedef ptrdiff_t         difference_type;

    template <class U> 
    struct rebind { typedef MMapAllocator<U> other; };

    MMapAllocator() {}

    MMapAllocator(const MMapAllocator&) {}

#if !defined(_MSC_VER)
    template <class U> 
    MMapAllocator(const MMapAllocator<U>&) {}
#endif

    ~MMapAllocator() {}

    pointer address(reference x) const { return &x; }
    const_pointer address(const_reference x) const { 
      return x;
    }

    pointer allocate(size_type n, const_pointer = 0) {
      void* p = malloc(n * sizeof(T));
      if (!p)
	THROW(bad_alloc());
      return static_cast<pointer>(p);
    }

    pointer reallocate(size_type n, const_pointer = 0) {
      void* p = malloc(n * sizeof(T));
      if (!p)
	THROW(bad_alloc());
      return static_cast<pointer>(p);
    }

    void deallocate(pointer p, size_type) { free(p); }

    size_type max_size() const { 
      return static_cast<size_type>(-1) / sizeof(T);
    }

    void construct(pointer p, const value_type& x) { 
      new(p) value_type(x); 
    }
    void destroy(pointer p) { p->~value_type(); }

  private:
    void operator=(const MMapAllocator&);

  };

  template<> class MMapAllocator<void>
  {
    typedef void        value_type;
    typedef void*       pointer;
    typedef const void* const_pointer;

    template <class U> 
    struct rebind { typedef MMapAllocator<U> other; };
  };


  template <class T>
  inline bool operator==(const MMapAllocator<T>&, 
			 const MMapAllocator<T>&) {
    return false;
  }

  template <class T>
  inline bool operator!=(const MMapAllocator<T>&, 
			 const MMapAllocator<T>&) {
    return true;
  }

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_MMAP_ALLOCATOR_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
