
#ifndef _DAMLDB_STATEMENT_HH
#define _DAMLDB_STATEMENT_HH

#if defined (sun)
#  pragma ident "@(#)$Id: Statement.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/Portability.hh"

#ifdef CCXX_NAMESPACES
namespace damldb {
#endif

  typedef long Flags;
  typedef unsigned long StatementID;
  typedef unsigned long ResourceID;

  /** @name Null Object Definitions */
  //@{
  /// The StatementID of the NULL Statement
  const unsigned long NULL_STATEMENT_ID = 0xffffffff;

  /// The StatementID of the NULL Statement
  const unsigned long NULL_RESOURCE_ID  = 0xffffffff;
  //@}

  /** @name Statement Flags Definitions */
  //@{
  /// Represents a valid Statement Record
  const unsigned int STATEMENT_FLAG_VALID   = 0x1;

  /// Represents a deleted Statement Record
  const unsigned int STATEMENT_FLAG_DELETED = 0x2;

  /// Represents statement's object is a literal object
  const unsigned int STATEMENT_FLAG_LITERAL_OBJECT = 0x4;
  //@}

  /** Statement is damldb representation of an RDF Statement
   */
  struct Statement {
    // StatementID is implicit from statement file offset
    Flags flags;
    ResourceID source;
    StatementID sourceNext;
    ResourceID subject;
    StatementID subjectNext;
    ResourceID predicate;
    StatementID predicateNext;
    ResourceID object;
    StatementID objectNext;
  };

  /** @name Statement Flags Definitions */
  //@{
  /// Represents a valid Resource Record 
  const int RESOURCE_FLAG_VALID = 0x1;

  /// Represents a deleted Resource Record
  const int RESOURCE_FLAG_DELETED = 0x2;

  /// Represents a literal resource
  const int RESOURCE_FLAG_LITERAL = 0x4;
  //@}

  /** Resource is damldb representation of an RDF Resource
   */
  struct Resource {
    // ResourceID is implicit from resource file offset
    Flags flags;
    StatementID sourceFirst;
    StatementID subjectFirst;
    StatementID predicateFirst;
    StatementID objectFirst;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif //!_DAMLDB_STATEMENT_HH

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:
