#ifndef _DAMLDB_TIMER_HH
#define _DAMLDB_TIMER_HH

#if defined (sun)
#  pragma ident "@(#)$Id: Timer.hh,v 1.2 2002/04/09 02:04:46 neves Exp $"
#endif

#include "damldb/OS.hh"
#include "damldb/Portability.hh"
#include "damldb/Time_Value.hh"

#include <stdexcept>

#ifdef CCXX_NAMESPACES
namespace damldb {
  USING(damldb);
#endif

  /**
     Timer is a simple stop-start timer.
   */
  class Timer
  {
  public:

    /// Default Constructor
    Timer() : running_(false) {}

    /** Reset the timer.
	@precondition isRunning'() == false
	@postcondition isRunning() == false
    */
    void reset()
    {
      if (running_)
	{
	  THROW(runtime_error("Timer is still running"));
	}

      start_.set(0,0);
      end_.set(0,0);
      total_.set(0,0);
      running_ = false;
    }

    /** Start the timer.
	@precondition isRunning'() == false
	@postcondition isRunning() == true 
	@postcondition this.start_ != Time_Value::zero
   */
    void start()
    {
      if (running_)
	{
	  THROW(runtime_error("Timer has already been started"));
	}
      start_ = OS::gettimeofday();
      running_ = true;
    }

    /** Stop the timer.
	@precondition isRunning'() == true
	@postcondition isRunning() == false
	@postcondition this.end_ != Time_Value::zero
	@postcondition this.start_ <= this.end_
	@postcondition this.total_ != Time_Value::zero
    */
    void stop()
    {
      if (! running_)
	{
	  THROW(runtime_error("Timer has not been started"));
	}
      end_ = OS::gettimeofday();
      total_ = end_ - start_;
      running_ = false;
    }

    /** Return the elapsed time.
	The elapsed time between start() and stop() method invocations.
	@precondition isRunning'() == true
     */
    const Time_Value& elapsed_time() const
    {
      if (running_)
	{
	  THROW(runtime_error("Timer is still running"));
	}
      return total_;
    }

    /** Return whether the timer is still running.
     */
    bool isRunning() const
    {
      return running_;
    }

  protected:

    // Starting time.
    Time_Value start_;

    // Ending time.
    Time_Value end_;

    // Total elapsed time.
    Time_Value total_;

    // Status of the timer.
    bool running_;
  };

#ifdef CCXX_NAMESPACES
} // namespace damldb
#endif

#endif /* !DAMLDB_TIMER_HH */

// Emacs
// Local Variables:
// mode: c++
// c-basic-offset: 2
// End:

