
#ifndef lint
static const char revid[] = 
    "@(#)$Id: support.c,v 1.2 2002/04/09 21:59:51 neves Exp $";
#endif // not lint

#include "damldb/support.h"

#include <stdio.h>
#include <stdlib.h>

/**@name Support Functions */

//@{
#if !defined(HAVE_ITOA)
/** Integer to ASCII conversion function.
    This function is supplied for those platforms that do not implement
    the 'itoa' function.

    @param value the integer value to convert [input].
    @param buf a character buffer that holds the ascii string result.
    @param radix the base value of the integer representation.
    @return a pointer to the computed ASCII null-terminated result.
 */
char* itoa(int value, char *buf, int radix)
{
  sprintf(buf, "%d", value);
  return buf;
}
#endif

#if !defined(HAVE_COMPRESS)
/** Compress (in-memory) the source buffer and place the results in the destination buffer.
    The package local implementation NEVER returns and terminates the program.

    @param dest the destination of the compress output.
    @param destLen the length of the destination buffer available.
    @param source the source buffer to be compressed.
    @param sourceLne the length of the source buffer.
    @return 0 if success, otherwise an error occurred.
 */
int compress(char* dest, unsigned long* destLen, const char* source, unsigned long sourceLen)
{
  fprintf(stderr, "%s function is missing\n", "compress");
  exit(EXIT_FAILURE);
}
#endif


#if !defined(HAVE_COMPRESS)
/** Uncompress (in-memory) the source buffer and place the results in the destination buffer.
    The package local implementation NEVER returns and terminates the program.

    @param dest the destination of the uncompressed output.
    @param destLen the length of the destination buffer available.
    @param source the source buffer to be uncompressed.
    @param sourceLne the length of the source buffer.
    @return 0 if success, otherwise an error occurred.
 */
int uncompress(char* dest, unsigned long* destLen, const char* source, unsigned long sourceLen)
{
  fprintf(stderr, "%s function is missing\n", "uncompress");
  exit(EXIT_FAILURE);
}
#endif

#if defined(_MSC_VER)
/** Translate the error code of the last Win32 API function invocation.
 */
char* win32_strerror()
{
  static char buf[1024];
  LPVOID lpMsgBuf;
  DWORD error = GetLastError();

  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		error,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) & lpMsgBuf,
		0,
		NULL);

  strncpy(buf, (char *) lpMsgBuf, sizeof(buf));
  LocalFree(lpMsgBuf);

  return buf;
}
#endif

/** A private dummy function to force an object file.
 */
static void dummy() {
}

//@}
